-----------------------------------------------------------------------------
-- $Id: MArray.hs,v 1.16 2001/03/30 04:54:28 qrczak Exp $
--
-- (c) The GHC Team 2000
--

module MArray ( 

	-- Class of mutable arrays in a given monad
	MArray {- a:*->*->*  e:*  m:*->* -} ( 
	   newArray,  -- :: Ix ix => (ix,ix) -> e -> m (a ix e)
	   newArray_, -- :: Ix ix => (ix,ix) -> m (a ix e)
	   readArray, -- :: Ix ix => a ix e -> ix -> m e
	   writeArray -- :: Ix ix => a ix e -> ix -> e -> m ()
	),

	module Ix,
	HasBounds {- a:*->*->* -} (
	   bounds  -- :: Ix ix => a ix e -> (ix,ix)
	),

	STArray,      -- instance HasBounds, MArray
	STUArray,     -- instance HasBounds, MArray (Char, Int, Word etc.)

	IOArray,      -- instance HasBounds, MArray
	IOUArray,     -- instance HasBounds, MArray (Char, Int, Word etc.)

	getAssocs,    -- :: (Ix ix, MArray a e m) => a s ix e -> m s [(ix,e)]
	getElems,     -- :: (Ix ix, MArray a e m) => a ix e -> m [e]
	indices,      -- :: (Ix ix, HasBounds a) => a ix e -> [ix]

 	mapArray,     -- :: (Ix ix, MArray a x m, MArray a y m) => 
		      --	   (x->y) -> a ix x -> m (a ix y)
	mapIndices,   -- :: (Ix ix, Ix iy, MArray a e m) => 
		      --	(ix,ix) -> (ix->iy) -> a iy e -> m (a ix e)

	newListArray, -- :: (Ix ix, MArray a e m) => (ix,ix) -> [e] -> m (a ix e)


	freeze,       -- :: (Ix ix, MArray a e m, IArray b e) => 
		      --		a ix e -> m (b ix e)
	unsafeFreeze, -- :: (Ix ix, MArray a e m, IArray b e) => 
		      --		a ix e -> m (b ix e)

        thaw,         --  :: (Ix ix, IArray a e, MArray b e m) => 
		      --		a ix e -> m (b ix e)
	unsafeThaw,   --  :: (Ix ix, IArray a e, MArray b e m) => 
		      --		a ix e -> m (b ix e)

        castSTUArray, -- :: STUArray s ix a -> ST s (STUArray s ix b)
        castIOUArray, -- :: IOUArray ix a -> IO (IOUArray ix b)

	-- legacy non-overloaded byte array interface
	newCharArray,
	newIntArray,
	newWordArray,
	newAddrArray,
	newFloatArray,
	newDoubleArray,
	newStablePtrArray,
	newInt8Array,
	newInt16Array,
	newInt32Array,
	newInt64Array,
	newWord8Array,
	newWord16Array,
	newWord32Array,
	newWord64Array,

	readCharArray,
	readIntArray,
	readWordArray,
	readAddrArray,
	readFloatArray,
	readDoubleArray,
	readStablePtrArray,
	readInt8Array,
	readInt16Array,
	readInt32Array,
	readInt64Array,
	readWord8Array,
	readWord16Array,
	readWord32Array,
	readWord64Array,

	writeCharArray,
	writeIntArray,
	writeWordArray,
	writeAddrArray,
	writeFloatArray,
	writeDoubleArray,
	writeStablePtrArray,
	writeInt8Array,
	writeInt16Array,
	writeInt32Array,
	writeInt64Array,
	writeWord8Array,
	writeWord16Array,
	writeWord32Array,
	writeWord64Array,
  ) where

import ArrayBase

import IOExts
import Addr
import Word
import Int
import IArray (indices)
import qualified IArray
import Ix
import ST
import PrelFloat
import PrelBase
import PrelGHC
import PrelPtr
import PrelStable

import Prelude hiding (read)

-----------------------------------------------------------------------------
-- Polymorphic non-strict mutable arrays (IO monad)

instance HasBounds IOArray where
   bounds = boundsIOArray

instance MArray IOArray e IO where
   newArray      = newIOArray
   newArray_ ixs = newIOArray ixs io_bot
   readArray     = readIOArray
   writeArray    = writeIOArray

io_bot :: a
io_bot = error "IOArray: undefined element"
	--ToDo: throw (ArrayException UndefinedElement)

-----------------------------------------------------------------------------
-- Flat unboxed mutable arrays (IO monad)

newtype IOUArray ix e = IOUArray (STUArray RealWorld ix e)
-- could be a type synonym, but IOUArray will probably make
-- more sense in error messages.

boundsOfIOUArray :: IOUArray ix e -> (ix,ix)
boundsOfIOUArray (IOUArray a) = boundsSTUArray a

instance HasBounds IOUArray where
   bounds = boundsOfIOUArray

instance MArray IOUArray Bool IO where
   newArray_ ixs = do { a <- stToIO (newBoolArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readBoolArray a i)
   writeArray (IOUArray a) i e = stToIO (writeBoolArray a i e)

instance MArray IOUArray Char IO where
   newArray_ ixs = do { a <- stToIO (newWideCharArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWideCharArray a i)
   writeArray (IOUArray a) i e = stToIO (writeWideCharArray a i e)

instance MArray IOUArray Int IO where
   newArray_ ixs = do { a <- stToIO (newIntArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readIntArray a i)
   writeArray (IOUArray a) i e = stToIO (writeIntArray a i e)

instance MArray IOUArray Word IO where
   newArray_ ixs = do { a <- stToIO (newWordArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWordArray a i)
   writeArray (IOUArray a) i e = stToIO (writeWordArray a i e)

instance MArray IOUArray Addr IO where
   newArray_ ixs = do { a <- stToIO (newAddrArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readAddrArray a i)
   writeArray (IOUArray a) i e = stToIO (writeAddrArray a i e)

instance MArray IOUArray (Ptr a) IO where
   newArray_ ixs = do { a <- stToIO (newPtrArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readPtrArray a i)
   writeArray (IOUArray a) i e = stToIO (writePtrArray a i e)

instance MArray IOUArray Float IO where
   newArray_ ixs = do { a <- stToIO (newFloatArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readFloatArray a i)
   writeArray (IOUArray a) i e = stToIO (writeFloatArray a i e)

instance MArray IOUArray Double IO where
   newArray_ ixs = do { a <- stToIO (newDoubleArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readDoubleArray a i)
   writeArray (IOUArray a) i e = stToIO (writeDoubleArray a i e)

instance MArray IOUArray (StablePtr a) IO where
   newArray_ ixs = do { a <- stToIO (newStablePtrArray ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readStablePtrArray a i)
   writeArray (IOUArray a) i e = stToIO (writeStablePtrArray a i e)

instance MArray IOUArray Int8 IO where
   newArray_ ixs = do { a <- stToIO (newInt8Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readInt8Array a i)
   writeArray (IOUArray a) i e = stToIO (writeInt8Array a i e)

instance MArray IOUArray Int16 IO where
   newArray_ ixs = do { a <- stToIO (newInt16Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readInt16Array a i)
   writeArray (IOUArray a) i e = stToIO (writeInt16Array a i e)

instance MArray IOUArray Int32 IO where
   newArray_ ixs = do { a <- stToIO (newInt32Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readInt32Array a i)
   writeArray (IOUArray a) i e = stToIO (writeInt32Array a i e)

instance MArray IOUArray Int64 IO where
   newArray_ ixs = do { a <- stToIO (newInt64Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readInt64Array a i)
   writeArray (IOUArray a) i e = stToIO (writeInt64Array a i e)

instance MArray IOUArray Word8 IO where
   newArray_ ixs = do { a <- stToIO (newWord8Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWord8Array a i)
   writeArray (IOUArray a) i e = stToIO (writeWord8Array a i e)

instance MArray IOUArray Word16 IO where
   newArray_ ixs = do { a <- stToIO (newWord16Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWord16Array a i)
   writeArray (IOUArray a) i e = stToIO (writeWord16Array a i e)

instance MArray IOUArray Word32 IO where
   newArray_ ixs = do { a <- stToIO (newWord32Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWord32Array a i)
   writeArray (IOUArray a) i e = stToIO (writeWord32Array a i e)

instance MArray IOUArray Word64 IO where
   newArray_ ixs = do { a <- stToIO (newWord64Array ixs); return (IOUArray a) }
   readArray  (IOUArray a) i   = stToIO (readWord64Array a i)
   writeArray (IOUArray a) i e = stToIO (writeWord64Array a i e)

freezeIOUArray :: Ix ix => IOUArray ix e -> IO (UArray ix e)
freezeIOUArray (IOUArray a) = stToIO (freezeSTUArray a)

{-# RULES
"freeze/IOUArray"   freeze = freezeIOUArray
 #-}

unsafeFreezeIOUArray :: Ix ix => IOUArray ix e -> IO (UArray ix e)
unsafeFreezeIOUArray (IOUArray a) = stToIO (freezeSTUArray a)

{-# RULES
"unsafeFreeze/IOUArray"   unsafeFreeze = unsafeFreezeIOUArray
 #-}

thawIOUArray :: Ix ix => UArray ix e -> IO (IOUArray ix e)
thawIOUArray a = do a' <- stToIO (thawSTUArray a); return (IOUArray a')

{-# RULES
"thaw/IOUArray"   thaw = thawIOUArray
 #-}

unsafeThawIOUArray :: Ix ix => UArray ix e -> IO (IOUArray ix e)
unsafeThawIOUArray a = do a' <- stToIO (thawSTUArray a); return (IOUArray a')

{-# RULES
"unsafeThaw/IOUArray"   unsafeThaw = unsafeThawIOUArray
 #-}

-----------------------------------------------------------------------------
-- standard array ops

newListArray :: (Ix ix, MArray a e m) => (ix,ix) -> [e] -> m (a ix e)
newListArray b elems = do
  a <- newArray_ b
  sequence_ [ writeArray a i e | (i,e) <- zip (range b) elems ]
  return a

mapArray :: (Ix ix, MArray a x m, MArray a y m) => (x->y) -> a ix x -> m (a ix y)
mapArray f a = do
  new <- newArray_ (bounds a)
  sequence_ [ do e <- readArray a i; writeArray new i (f e) | i <- indices a ]
  return new
  
getElems  :: (Ix ix, MArray a e m) => a ix e -> m [e]
getElems a = mapM (readArray a) (indices a)

mapIndices :: (Ix ix, Ix iy, MArray a e m) => 
		(ix,ix) -> (ix->iy) -> a iy e -> m (a ix e)
mapIndices b f a = do
  new <- newArray_ b
  sequence_ [ do e <- readArray a (f i); writeArray new i e | i <- range b ]
  return new

castSTUArray :: STUArray s ix a -> ST s (STUArray s ix b)
castSTUArray (STUArray l u contents) = return (STUArray l u contents)

castIOUArray :: IOUArray ix a -> IO (IOUArray ix b)
castIOUArray (IOUArray (STUArray l u contents))
   = return (IOUArray (STUArray l u contents))
